<?php

namespace Drupal\date_range_availability;

use Twig\TwigFunction;
use Drupal\Component\Datetime\TimeInterface;
use Twig\Extension\AbstractExtension;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a service to determine node availability based on date range field.
 *
 * This service checks the availability of a node by comparing the current date
 * with the start and end dates of a specified date range field. It returns
 * the availability status (e.g., "Coming Soon", "Available", "Unavailable")
 * along with a CSS class for styling.
 */
class NodeAvailabilityService extends AbstractExtension {

  /**
   * Constructs a new NodeAvailabilityService object.
   *
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  protected TimeInterface $time;

  public function __construct(TimeInterface $time) {
    $this->time = $time;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('datetime.time')
    );
  }

  /**
   * Custom twig functions.
   */
  public function getFunctions() {
    return [
      new TwigFunction('node_availability', [$this, 'getNodeAvailability']),
    ];
  }

  /**
   * Retrieves node state based on the date range.
   *
   * @param int $node
   *   The node.
   * @param string $field_date
   *   The date range field of the node.
   *
   * @return array
   *   Returns an array with the current state and a CSS class for styling.
   */
  public function getNodeAvailability($node, $field_date) {
    $node_availability = '';
    $node_availability_class = '';
    // Load the node.
    $expire_date = 0;
    $start_date = 0;
    $start_date = $node->get($field_date)->value ? strtotime($node->get($field_date)->value) : 0;
    $expire_date = $node->get($field_date)->end_value ? strtotime($node->get($field_date)->end_value) : 0;
    // Return early if the node is not found or not published.
    if (!$node || !$node->isPublished()) {
      return [
        'label' => t('Unavailable'),
        'label_class' => 'unavailable-label',
      ];
    }
    // Get the current time.
    $current_date = $this->time->getCurrentTime();
    // Get the start and end dates from the field.
    $start_date = $node->get($field_date)->value ? strtotime($node->get($field_date)->value) : 0;
    $expire_date = $node->get($field_date)->end_value ? strtotime($node->get($field_date)->end_value) : 0;

    // Determine the node availability state.
    if ($expire_date != 0 && $current_date < $start_date) {
      $node_availability_class = 'soon-label';
      $node_availability = t('Coming Soon');
    }
    elseif ($expire_date != 0 && $current_date > $start_date && $current_date < $expire_date) {
      $node_availability_class = 'available-label';
      $node_availability = t('Available');
    }
    elseif ($expire_date != 0 && $current_date > $expire_date) {
      $node_availability_class = 'unavailable-label';
      $node_availability = t('Unavailable');
    }

    return [
      'label' => $node_availability,
      'label_class' => $node_availability_class,
    ];

  }

}
