<?php

namespace Drupal\date_range_availability\Plugin\views\field;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Drupal\date_range_availability\NodeAvailabilityService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Views field handler to display node state based on a date range field.
 *
 * This field handler checks the availability of a node by comparing the current
 * date with the start and end dates of a specified date range field.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("node_availability")
 */
class NodeAvailability extends FieldPluginBase {

  /**
   * The node availability service.
   *
   * @var \Drupal\date_range_availability\NodeAvailabilityService
   */
  protected NodeAvailabilityService $nodeAvailabilityService;

  /**
   * Constructs a NodeAvailability object.
   *
   * @param \Drupal\date_range_availability\NodeAvailabilityService $nodeAvailabilityService
   *   The node availability service.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(NodeAvailabilityService $nodeAvailabilityService, array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->nodeAvailabilityService = $nodeAvailabilityService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('date_range_availability.node_availability_service'),
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * {@inheritdoc}
   *
   * This method is intentionally left empty.
   */
  public function query() {
    // Do nothing -- to override the parent query.
  }

  /**
   * Defines the available options for this field handler.
   *
   * @return array
   *   An array of options for this field handler.
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    // Add a custom option for the date range field machine name.
    $options['field_date'] = ['default' => ''];
    return $options;
  }

  /**
   * Provides the options form for this field handler.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    // Add a text field for the date range field machine name.
    $form['field_date'] = [
      '#title' => t('Machine name of the date range field'),
      '#type' => 'textfield',
      '#description' => t('Enter the machine name of the date field (e.g., "field_date").'),
      '#default_value' => $this->options['field_date'],
      '#required' => TRUE,
    ];

    // Call the parent method to include additional options.
    parent::buildOptionsForm($form, $form_state);
  }

  /**
   * Renders the field content.
   *
   * @param \Drupal\views\ResultRow $values
   *   The result row object containing the data for this row.
   *
   * @return string
   *   The rendered availability label.
   */
  public function render(ResultRow $values) {
    // Get the node entity from the result row.
    $node = $this->getEntity($values);

    // Use the node availability service to determine the node's availability.
    $node_availability = $this->nodeAvailabilityService->getNodeAvailability($node, $this->options['field_date']);

    // Return the availability label.
    return $node_availability['label'];
  }

}
